@@ -1,4 +1,4 @@
-;;;; $Id: protocol.lisp,v 1.5 2004/03/18 21:57:25 ehuelsmann Exp $
+;;;; $Id: protocol.lisp,v 1.11 2004/08/06 13:08:09 bmastenbrook Exp $
 ;;;; $Source: /project/cl-irc/cvsroot/cl-irc/protocol.lisp,v $
 
 ;;;; See LICENSE for licensing information.
@@ -29,14 +29,14 @@
     :initarg :server-name
     :accessor server-name
     :initform "Unknown server")
-   (server-socket
-    :initarg :server-socket
-    :accessor server-socket
-    :documentation "Socket used to talk to the IRC server.")
    (server-stream
     :initarg :server-stream
     :accessor server-stream
     :documentation "Stream used to talk to the IRC server.")
+   (server-socket
+    :initarg :server-socket
+    :accessor server-socket
+    :initform nil)
    (client-stream
     :initarg :client-stream
     :accessor client-stream
@@ -79,15 +79,15 @@ this stream.")
 
 (defun make-connection (&key (user nil)
                              (server-name "")
-                             (server-socket nil)
                              (server-stream nil)
+			(server-socket nil)
                              (client-stream t)
                              (hooks nil))
   (let ((connection (make-instance 'connection
                                    :user user
                                    :server-name server-name
-                                   :server-socket server-socket
                                    :server-stream server-stream
+				   :server-socket server-socket
                                    :client-stream client-stream)))
     (dolist (hook hooks)
       (add-hook connection (car hook) (cadr hook)))
@@ -123,7 +123,13 @@ input."
   (let ((stream (server-stream connection)))
     (and (streamp stream)
          (open-stream-p stream))))
-  
+
+(define-condition invalidate-me (condition)
+  ((socket :initarg :socket
+           :reader invalidate-me-socket)
+   (condition :initarg :condition
+              :reader invalidate-me-condition)))
+
 (defmethod read-message ((connection connection))
   (let ((read-more-p t))
     (handler-case
@@ -134,7 +140,10 @@ input."
                 (format *debug-stream* "~A" (describe message)))
               (irc-message-event message)
               message))) ; needed because of the "loop while" in read-message-loop
-      (stream-error () (setf read-more-p nil)))))
+      (stream-error (c) (setf read-more-p nil)
+                    (signal 'invalidate-me :socket
+                            (server-socket connection)
+                            :condition c)))))
 
 (defvar *process-count* 0)
 
@@ -142,28 +151,38 @@ input."
   #+allegro (mp:process-run-function name function)
   #+cmu (mp:make-process function :name name)
   #+lispworks (mp:process-run-function name nil function)
-  #+sb-thread (sb-thread:make-thread function))
-  
+  #+sb-thread (sb-thread:make-thread function)
+  #+openmcl (ccl:process-run-function name function)
+  #+armedbear (ext:make-thread function))
+
 (defmethod start-background-message-handler ((connection connection))
   "Read messages from the `connection', parse them and dispatch
 irc-message-event on them. Returns background process ID if available."
   (flet ((do-loop () (read-message-loop connection)))
     (let ((name (format nil "irc-hander-~D" (incf *process-count*))))
-      #+(or allegro cmu lispworks sb-thread)
+      #+(or allegro cmu lispworks sb-thread openmcl armedbear)
       (start-process #'do-loop name)
       #+(and sbcl (not sb-thread))
-      (sb-sys:add-fd-handler (sb-bsd-sockets:socket-file-descriptor
-			      (server-socket connection))
+      (sb-sys:add-fd-handler (sb-sys:fd-stream-fd
+			      (server-stream connection))
 			     :input (lambda (fd)
 				      (declare (ignore fd))
-				      (read-message connection))))))
+                                      (handler-case
+                                          (read-message connection)
+                                        (invalidate-me (c)
+                                          (sb-sys:invalidate-descriptor
+                                           (invalidate-me-socket c))
+                                          (format t "Socket closed: ~A~%"
+                                                  (invalidate-me-condition c)))))))))
 
 (defun stop-background-message-handler (process)
   "Stops a background message handler process returned by the start function."
     #+cmu (mp:destroy-process process)
     #+allegro (mp:process-kill process)
     #+sb-thread (sb-thread:destroy-thread process)
-    #+lispworks (mp:process-kill process))
+    #+lispworks (mp:process-kill process)
+    #+openmcl (ccl:process-kill process)
+    #+armedbear (ext:destroy-thread process))
 
 (defmethod read-message-loop ((connection connection))
   (loop while (read-message connection)))
@@ -189,7 +208,7 @@ server, via the `connection'."
 (defmethod get-hooks ((connection connection) (class symbol))
   "Return a list of all hooks for `class'."
   (gethash class (hooks connection)))
-        
+
 (defmethod add-hook ((connection connection) class hook)
   "Add `hook' to `class'."
   (setf (gethash class (hooks connection))
@@ -237,8 +256,7 @@ between the two users.")))
         (format stream "with ~A@~A"
                 (nickname (user object))
                 (hostname (user object)))
-                
-        "")))
+      "")))
 
 (defun make-dcc-connection (&key (user nil)
                                  (remote-address nil)
@@ -252,7 +270,14 @@ between the two users.")))
                    :stream (sb-bsd-sockets:socket-make-stream socket :input t :output t :buffering :none)
                    :socket socket
                    :output-stream t))
-  #-sbcl
+  #+openmcl
+  (let ((socket-stream (ccl:make-socket :remote-host remote-address
+                                        :remote-port remote-port)))
+    (make-instance 'dcc-connection
+                   :user user
+                   :stream socket-stream
+                   :output-stream output-stream))
+  #-(or openmcl sbcl)
   (warn "make-dcc-connection not supported for this implementation."))
 
 (defgeneric dcc-close (connection))
@@ -315,7 +340,7 @@ channel.  Rather, the number returned from the LIST command gets stuck
 in there so the user of this library can use it for searching
 channels, for instance.  If the value is NIL then the slot has not
 been populated by a LIST command.")))
-    
+
 (defmethod print-object ((object channel) stream)
   "Print the object for the Lisp reader."
   (print-unreadable-object (object stream :type t :identity t)
@@ -325,7 +350,7 @@ been populated by a LIST command.")))
   "Normalize `string' so that it represents an all-downcased channel
 name."
   (string-downcase string))
-  
+
 (defun make-channel (&key (name "")
                           (topic "")
                           (modes nil)
@@ -439,7 +464,7 @@ nickname."
 (defgeneric remove-all-users (connection))
 (defgeneric remove-user (object user))
 (defgeneric remove-user-everywhere (connection user))
-(defgeneric find-or-make-user (connection nickname 
+(defgeneric find-or-make-user (connection nickname
                                           &key username hostname realname))
 (defgeneric change-nickname (connection user new-nickname))
 
@@ -471,8 +496,8 @@ known."
 
 (defmethod remove-channel ((channel channel) (user user))
   "Remove `channel' from `user'."
-  (warn 
-   (concatenate 'string 
+  (warn
+   (concatenate 'string
                 "use of depricated API (remove-channel channel user): "
                 "(remove-channel user channel) is now preferred"))
   (remove-channel user channel))
@@ -567,36 +592,35 @@ may be already be on."
 
 (defclass irc-error-reply (irc-message) ())
 
-(defun intern-message-symbol (prefix name)
-  "Intern based on symbol-name to support case-sensitive mlisp"
-  (intern
-   (concatenate 'string
-		(symbol-name prefix)
-		"-"
-		(symbol-name name)
-		"-"
-		(symbol-name '#:message))))
-  
-(defmacro define-irc-message (command)
-  (let ((name (intern-message-symbol :irc command)))
-    `(progn
-      (defmethod find-irc-message-class ((type (eql ,command)))
-	(find-class ',name))
-      (export ',name)
-      (defclass ,name (irc-message) ()))))
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defun intern-message-symbol (prefix name)
+    "Intern based on symbol-name to support case-sensitive mlisp"
+    (intern
+     (concatenate 'string
+		  (symbol-name prefix)
+		  "-"
+		  (symbol-name name)
+		  "-"
+		  (symbol-name '#:message))))
 
-(defun create-irc-message-classes (class-list)
-  (dolist (class class-list)
-    (eval (list 'define-irc-message class)))) ; argh.  eval.
+  (defun define-irc-message (command)
+    (let ((name (intern-message-symbol :irc command)))
+      `(progn
+	(defmethod find-irc-message-class ((type (eql ,command)))
+	  (find-class ',name))
+	(export ',name)
+	(defclass ,name (irc-message) ())))))
+
+(defmacro create-irc-message-classes (class-list)
+  `(progn ,@(mapcar #'define-irc-message class-list)))
 
 ;; should perhaps wrap this in an eval-when?
-(create-irc-message-classes (remove-duplicates
-                             (mapcar #'second *reply-names*)))
-(create-irc-message-classes '(:privmsg :notice :kick :topic :error
-                              :mode :ping :nick :join :part :quit :kill
-			      :pong :invite))
+(create-irc-message-classes #.(remove-duplicates (mapcar #'second *reply-names*)))
+(create-irc-message-classes (:privmsg :notice :kick :topic :error :mode :ping
+			     :nick :join :part :quit :kill :pong :invite))
 
 (defmethod find-irc-message-class (type)
+  (declare (ignore type))
   (find-class 'irc-message))
 
 (defmethod client-log ((connection connection) (message irc-message) &optional (prefix ""))
@@ -624,28 +648,29 @@ may be already be on."
     :initarg :ctcp-command
     :accessor ctcp-command)))
 
-(defclass standard-ctcp-message (ctcp-mixin message) ())
+(defclass standard-ctcp-message (ctcp-mixin irc-message) ())
 
 (defgeneric find-ctcp-message-class (type))
 
-(defmacro define-ctcp-message (ctcp-command)
-  (let ((name (intern-message-symbol :ctcp ctcp-command)))
-    `(progn
-      (defmethod find-ctcp-message-class ((type (eql ,ctcp-command)))
-	(find-class ',name))
-      (export ',name)
-      (defclass ,name (ctcp-mixin irc-message) ()))))
-
-(defun create-ctcp-message-classes (class-list)
-  (dolist (class class-list)
-    (eval (list 'define-ctcp-message class)))) ; argh.  eval.  must go away.
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defun define-ctcp-message (ctcp-command)
+    (let ((name (intern-message-symbol :ctcp ctcp-command)))
+      `(progn
+	(defmethod find-ctcp-message-class ((type (eql ,ctcp-command)))
+	  (find-class ',name))
+	(export ',name)
+	(defclass ,name (ctcp-mixin irc-message) ())))))
+  
+(defmacro create-ctcp-message-classes (class-list)
+  `(progn ,@(mapcar #'define-ctcp-message class-list)))
 
 ;; should perhaps wrap this in an eval-when?
-(create-ctcp-message-classes '(:action :source :finger :ping
+(create-ctcp-message-classes (:action :source :finger :ping
                                :version :userinfo :time :dcc-chat-request
                                :dcc-send-request))
 
 (defmethod find-ctcp-message-class (type)
+  (declare (ignore type))
   (find-class 'standard-ctcp-message))
 
 (defmethod client-log ((connection connection) (message ctcp-mixin) &optional (prefix ""))
